export declare type AutocompleteUiOffsetBounds = {
    top: number;
    left: number;
    height: number;
};

export declare enum AutocompleteActions {
    CLOSE = 0,
    SELECT = 1,
    MOVE_SELECTION_DOWN = 2,
    MOVE_SELECTION_UP = 3,
}

export declare enum SlackShortcutCommandType {
    CANVAS = "insert-canvas",
    AUDIO = "insert-audio",
    VIDEO = "insert-video",
    FILE = "insert-file",
    WORKFLOW = "insert-workflow",
    CODE = "inline-code",
    CODEBLOCK = "insert-codeblock",
    BLOCKQUOTE = "insert-blockquote",
    PLAIN = "insert-plain",
    H1 = "insert-h1",
    H2 = "insert-h2",
    H3 = "insert-h3",
    IMAGE = "insert-image",
    TABLE = "insert-table",
    CHECKLIST = "insert-checklist",
    BULLET_LIST = "insert-bullet-list",
    ORDERED_LIST = "insert-ordered-list",
    DIVIDER = "insert-divider",
}

export declare enum TemplateType {
    TEXT = "text",
    USER = "user",
    CHANNEL = "channel",
    DATE = "date",
}

export declare enum SlackObjectType {
    CHANNEL = 0,
    USER = 1,
    EMOJI = 2,
    FILE = 4,
    UNFURL = 5,
    DOCUMENT = 6,
    LINK = 7,
    DOCUMENT_USER_ACCESS = 9,
    PENDING_URL = 10,
    GENERIC = 11,
    TEMPLATE = 12,
    BACKGROUND = 13,
}

declare type SlackBaseStub = {
    type: Exclude<
        SlackObjectType,
        | SlackObjectType.USER
        | SlackObjectType.EMOJI
        | SlackObjectType.UNFURL
        | SlackObjectType.DOCUMENT
        | SlackObjectType.DOCUMENT_USER_ACCESS
        | SlackObjectType.PENDING_URL
        | SlackObjectType.TEMPLATE
        | SlackObjectType.BACKGROUND
    >;
    id: string;
};

export declare type SlackUserStub = {
    type: SlackObjectType.USER;
    id: string;
    memberHasExplicitAccess?: boolean;
};

export declare type SlackEmojiStub = {
    type: SlackObjectType.EMOJI;
    id: string;
    teamId: string;
};

export declare type SlackUnfurlStub = {
    type: SlackObjectType.UNFURL;
    id: string;
    channelId: string;
    teamId: string;
    appId?: string;
    isDeleted?: boolean;
};

export declare type SlackDocumentStub = {
    type: SlackObjectType.DOCUMENT;
    id: string;
    sectionId?: string;
    currentThreadId?: string;
    isCreatorView?: boolean;
};

export declare type SlackDocumentUserAccessStub = {
    type: SlackObjectType.DOCUMENT_USER_ACCESS;
    id: string;
    threadOrFileId: string;
};

export declare type SlackGenericStub = {
    type: SlackObjectType.GENERIC;
    id: string;
};

export declare type SlackTemplateStub = {
    type: SlackObjectType.TEMPLATE;
    id: string;
    name?: string;
    value?: string;
    templateType?: TemplateType;
};

export declare type SlackPendingUrlPlaceholderDetails = {
    pendingFileId?: string;
    width: number;
    height: number;
    maxWidth?: number;
    maxHeight?: number;
    isSlackDocument?: boolean;
    isImage?: boolean;
    loading?: boolean;
    fileSizeError?: boolean;
    onUploadClick?: () => void;
    fileSizeLimitMB?: number;
    allowUpload?: boolean;
};

declare type SlackPendingUrlStub = {
    type: SlackObjectType.PENDING_URL;
    id: string;
    placeholderDetails?: SlackPendingUrlPlaceholderDetails;
};

export declare type SlackObjectStub =
    | SlackBaseStub
    | SlackUserStub
    | SlackEmojiStub
    | SlackUnfurlStub
    | SlackDocumentStub
    | SlackDocumentUserAccessStub
    | SlackLinkStub
    | SlackGenericStub
    | SlackPendingUrlStub
    | SlackTemplateStub
    | SlackBackgroundStub;

export declare type SlackChannel = SlackBaseStub & {
    type: SlackObjectType.CHANNEL;
    name: string;
    isPrivate: boolean;
    workspaceId: string;
    isOrgShared?: boolean;
    isExtShared?: boolean;
    isNonExistent?: boolean;
};

export declare type SlackUser = SlackUserStub & {
    displayName: string;
    teamId: string;
    profileUrl: string;
    realName?: string;
    image48?: string;
    mergedUserId?: string;
};

export declare type SlackEmoji = SlackEmojiStub & {
    displayUrl: string;
    displayUrlMedium?: string;
    displayUrlLarge?: string;
    displayUrlXLarge?: string;
    nonAnimatedUrlSmall?: string;
    nonAnimatedUrlMedium?: string;
    nonAnimatedUrlLarge?: string;
    nonAnimatedUrlXLarge?: string;
    unicode?: string;
};

export declare type SlackUnfurl = SlackUnfurlStub & {
    inlinePreview: string;
    unfurlDidFail: boolean;
    isUnfurling: boolean;
    isEphemeral?: boolean;
    label?: string;
    faviconUrl?: string;
    emoji?: string;
    permalink?: string;
    isSavedForLater?: boolean;
    unfurlSucceeded?: boolean;
};

declare type SlackLinkStub = {
    type: SlackObjectType.LINK;
    id: string;
    url: string;
};

export declare type SlackLink = SlackLinkStub;

export declare type SlackFile = SlackBaseStub & {
    type: SlackObjectType.FILE;
    altText: string;
    name: string;
    mimeType?: string;
    thumbUrl?: string;
    width?: number;
    height?: number;
    permalink?: string;
    sourceFile?: Blob;
    isDeleted?: boolean;
    isNotFound?: boolean;
};

declare type SlackBackgroundStub = {
    type: SlackObjectType.BACKGROUND;
    id: string;
};

export declare type SlackBackground = SlackBackgroundStub;

export declare type SlackDocument = SlackDocumentStub & {
    name: string;
    richTextName?: string;
    isSameDocument?: boolean;
    permalink?: string;
    pendingCreation?: boolean;
    shouldShowDocument?: boolean;
    isDeleted?: boolean;
};

export declare type SlackDocumentUserAccess = SlackDocumentUserAccessStub & {
    access: string;
    mergedDocumentUserAccessId?: string;
    isExplicitlySharedWith?: boolean;
};

export declare type SlackGeneric = SlackGenericStub;

export declare type SlackTemplate = SlackTemplateStub;

export declare type AutocompleteResponse = {
    results: SlackObject[];
    moreResultsPossible: boolean;
};

declare type SlackPendingUrl = SlackPendingUrlStub;

export declare type SlackObject =
    | SlackPendingUrl
    | SlackChannel
    | SlackUser
    | SlackEmoji
    | SlackFile
    | SlackUnfurl
    | SlackDocument
    | SlackLink
    | SlackDocumentUserAccess
    | SlackGeneric
    | SlackTemplate
    | SlackBackground;

export declare enum GetSlackObjectFailureReason {
    NOT_FOUND = 0,
}

export declare type GetSlackObjectFailure = {
    reason: GetSlackObjectFailureReason;
    stubs: SlackObjectStub[];
};

export declare type GetSlackObjectsResponse = {
    objects: SlackObject[];
    failures?: GetSlackObjectFailure[];
};

export declare type GetSlackEmojiByUnicodeStringResponse = {
    emojis: SlackEmoji[];
};

export declare type GetSlackTextyHtmlResponse = {
    html: string;
};

export declare type GetSectionAnchorLinkResponse = {
    url: string;
};

export declare type CreateSlackCanvasResponse = {
    fileId: string;
};

export declare type SlackRichTextApi = {
    type: string;
    elements: {}[];
};

export interface AutocompleteUiProvider {
    /**
     * Performs a search with the query and triggers an Slack autocomplete
     * popover if there are any results on the bounds specified.
     * If there are no results, this method will close the modal.
     * @param query   - the search query, eg "Ke"
     * @param trigger - which symbol triggered the autocomplete
     * @param types   - a list of slack object types to search
     * @param bounds  - the location to display the autocomplete modal relative
     *                  to the embedded space document
     */
    updateAutocompleteSearchQuery(
        query: string,
        trigger: string,
        types: SlackObjectType[],
        bounds: AutocompleteUiOffsetBounds
    ): Promise<boolean>;
    /**
     * Performs an auxiliary action on the autocomplete component
     * such as closing, moving arrow up, etc.
     * @param action  - the action enum
     */
    performAutocompleteAction(action: AutocompleteActions): void;
    /**
     * Performs a shortcut action on the autocomplete component
     * that leads to a callback on the Slack side
     * @param shortcutCommand  - the SlackShortcutCommandType enum
     */
    handleAutocompleteCommand(shortcutCommand: SlackShortcutCommandType): void;
    setDelegate(delegate: AutocompleteUiProviderDelegate): void;
}

export interface AutocompleteUiProviderDelegate {
    /**
     * Slack -> Quip call to return the Slack object that was selected
     * during the autocomplete.
     * TODO: Remove resultId after some time
     * @param resultId - the id of the result object, still present to support
     *                   backward compatibility.
     * @param slackObject - the result object
     */
    updateAutocompleteResponse(
        resultId: string,
        slackObject?: SlackObject
    ): void;
    didSelectSlackObject(resultId: string, slackObject?: SlackObject): void;
    /**
     * Slack -> Quip call to return the Shortcut object that was selected
     * during the autocomplete.
     * @param SlackShortcutCommandType - the result shortcut Id
     */
    didSelectShortcutCommand(shortcutCommand?: SlackShortcutCommandType): void;
}

export declare enum CommentLevel {
    WRITE = 0,
    READ = 1,
    NONE = 2,
}

export declare type CommentUIBounds = {
    left: number;
    top: number;
    right: number;
    bottom: number;
    width: number;
    height: number;
};

export declare enum CommentFocusTarget {
    NONE = "none",
    THREAD_ROOT = "thread_root",
    THREAD_COMPOSER = "thread_composer",
}

export interface CommentsUiProvider {
    openCommentsThread(
        commentThreadId: string,
        params?: {
            focusTarget?: CommentFocusTarget;
        }
    ): Promise<boolean>;
    openInlineCommentsThread(params: {
        commentThreadId: string;
        bounds: CommentUIBounds | null;
        focusTarget?: CommentFocusTarget;
    }): Promise<boolean>;
    closeCommentsThread(commentThreadId: string): void;
    setDelegate(delegate: CommentsUiProviderDelegate): void;
}

export interface CommentsUiProviderDelegate {
    /**
     * The comments thread was closed, indicate to Quip that the "cover" over
     * the document should be hidden.
     */
    commentsThreadDidClose(
        commentThreadId: string,
        discardComment: boolean
    ): void;
}

export declare type CreateDocumentOptions = {
    title: string;
    threadTempId: string;
    docTempId: string;
    cannedDoc?: {
        name: string;
        variables?: {
            huddleNotes?: {
                title: string;
                unfurl: SlackUnfurlStub;
            };
        };
    };
    context?: string;
};

export declare enum ShortcutModifier {
    NONE = 0,
    SHIFT = 1,
    ALT = 2,
    META = 4,
    CTRL = 8,
    MOD = 16,
}

export declare enum SectionInsertionType {
    IMAGE = "image",
    HORIZONTAL_RULE = "horizontal-rule",
    CHECKLIST = "checklist",
    NUMBERED_LIST = "numbered-list",
    BULLETED_LIST = "bullet-list",
    TABLE = "table",
    CODE = "code",
    TEXT_H1_STYLE = "large-heading",
    TEXT_H2_STYLE = "medium-heading",
    TEXT_H3_STYLE = "small-heading",
    TEXT_PLAIN_STYLE = "plain",
    BLOCKQUOTE = "blockquote",
}

export interface EditorUiConnector {
    setDelegate(delegate: EditorUiConnectorDelegate): void;
    onTemplateControlCreated(): void;
}

export interface EditorUiConnectorDelegate {
    performActionForTemplateControl(args: TemplateControlActionArgs): void;
}

export declare enum TemplateControlAction {
    INSERT = "insert",
    EDIT = "edit",
    REMOVE = "remove",
    SHOW_CREATION_POPOVER = "show_creation_popover",
    HIGHLIGHT = "highlight",
}

export declare type TemplateControlActionArgs =
    | {
          action: TemplateControlAction.SHOW_CREATION_POPOVER;
      }
    | {
          action: TemplateControlAction.INSERT;
          name: string;
          type: TemplateType;
      }
    | {
          action: TemplateControlAction.EDIT;
          controlIds: string[];
      }
    | {
          action: TemplateControlAction.REMOVE;
          controlIds: string[];
      }
    | {
          action: TemplateControlAction.HIGHLIGHT;
          controlIds: string[];
          on: boolean;
      };

declare type DocBaseEmbedOptions = {
    /** If true, annotations will be left aligned in all sizes, unless
		rightAlignAnnotations is set to true */
    leftAlignAnnotations?: boolean;
    /** If true, annotations will be right aligned in all sizes */
    rightAlignAnnotations?: boolean;
    /** If true, annnotations will be allowed when edits are disabled */
    allowCommentsCreateWhenEditsDisabled?: boolean;
};

export declare enum NavigationTypeMetric {
    IN_APP_USER_INITIATED = 0,
    PERMALINK = 1,
    POPOUT = 2,
}

export declare type DocStaticEmbedOptions = {
    /**
     * Fast loading but lower-fidelity (server-rendered) output that can be used
     * for quick previews.
     */
    mode: "static";
    /**
     * Map from (hyphenated) CSS property name to values, which will be set
     * on the root DOM element that contains the embedded document.
     */
    style?: Record<string, string>;
    /**
     * Makes a best effort to hide the heading section that contains the
     * document title.
     */
    hideTitle?: boolean;
    /**
     * Hides the default content that is inserted into a document when first
     * created.
     */
    hideDefaultContent?: boolean;
    /**
     * Disables scrolling of the document content if it is longer than the
     * container.
     */
    disableScrolling?: boolean;
} & DocBaseEmbedOptions;

export declare type DocEditorEmbedOptions = {
    /** Interactive (editable) document. */
    mode: "editor" | "canvas" | "flexpane" | "preview";
    /** If true, the document will be loaded in read-only mode. */
    disableEditing?: boolean;
    /** Comment level allowed. Possible options Write, Read, None */
    commentLevel?: CommentLevel;
    /** If true, the document will not display the collapsed state icon. */
    disableCollapsedStateGutter?: boolean;
    /** If true, the document will not display the collapsed state icon for locked sections. */
    disableCollapsedStateGutterWhenLocked?: boolean;
    /** ID of the section that will be focused when the doc is embedded. */
    focusSectionId?: string;
    /** ID of the commment thread that will be focused
     * when the doc is embedded. */
    focusCommentThreadId?: string;
    /** Scroll position to restore. */
    scrollPosition?: DocScrollPosition;
    /** Unset defaults to IN_APP_USER_INITIATED */
    navigationTypeMetric?: NavigationTypeMetric;
    /** For canvas */
    fullBleed?: boolean;
    /** For canvas */
    transparent?: boolean;
    /** For adding an embedded header */
    embeddedHeader?: boolean;
    /** For adding an optimistic render area */
    hasOptimisticRenders?: boolean;
    /**
     * For canvas. The ON value of this flag is only effective the first time a
     * newly-created spaceship gets embedded. The reason is that the "Untitled"
     * section is inserted only once by the client after creation.
     */
    skipUntitledOnNewDoc?: boolean;
    /** For the insertion toolbar, we would like the editor to autoscroll if the
     * user is typing in a section that is below the insertion toolbar. This
     * number should be the height of the toolbar plus its bottom margin.
     */
    visibleBottomOffset?: number;
    /**
     * Override vertical top margin
     */
    verticalMarginTop?: number;
    /**
     * Override vertical bottom margin
     */
    verticalMarginBottom?: number;
    /**
     * Option to bookmark unfurls by default
     */
    unfurlsBookmarkedByDefault?: boolean;
    /**
     * Option to add attributes to editor dom node
     */
    editorDomAttributes?: {
        [key: string]: string | number | boolean;
    };
    /**
     * Version id to load the document
     */
    version?: string;
    /**
     * Version sequence to load the document
     */
    sequence?: number;
    /**
     * Placeholder title when the document is empty
     */
    titlePlaceholder?: string;
    /**
     * Placeholder text when the document is empty
     */
    textPlaceholder?: string;
    /**
     * Hide default section text if the document isn't empty (ie. don't display
     * "Type anything..." when the document isn't empty)
     */
    hideDefaultSectionTextWhenNotEmpty?: boolean;
    /**
     * Show default section text if the document is empty and not focused
     */
    displayDefaultSectionTextWhenUnfocusedAndEmpty?: boolean;
    /**
     * Options for showing different template elements
     */
    templateOptions: TemplateOptions;
    /** Minimum time between background calls. */
    backgroundMinDelay?: number;
    /** Maximum time between background calls. */
    backgroundMaxDelay?: number;
    /** Save delay for entity changes. */
    backgroundSaveDelay?: number;
    /** Save delay for entity attribute changes (i.e. not content). */
    backgroundAttributeSaveDelay?: number;
    /** Save delay for document title section changes */
    backgroundTitleSaveDelay?: number;
    /**
     * Save delay for entity changes when there are no
     * other viewers.
     */
    backgroundNoViewersSaveDelay?: number;
    /** Backoff to add for each active viewer -- 0 means no backoff */
    backgroundPerViewerBackoff?: number;
    /** Backoff to add for each active editor -- 0 means no backoff */
    backgroundPerEditorBackoff?: number;
    /** How long after section lock is lost to keep tracking editor status */
    backgroundEditorBackoffDebounce?: number;
    /** Don't allow file uploads to canvas when file uploads for slack connect
     *  are disabled */
    isSlackConnectFileSharingBlocked?: boolean;
    /** Upload permission setting:
     * disable_all
     * disable_all_except_images
     */
    slackDisableFileUploadsPref?: string;
    disableSlackSectionContextMenus?: boolean;
    disableAnimateLayoutDrag?: boolean;
} & DocBaseEmbedOptions;

export declare type TemplateOptions = {
    allowTemplateOverlay: boolean;
    showMeetingNotesOption?: boolean;
    linkToHelpCenter?: string;
    showLinkToHelpCenterInOverlay?: boolean;
};

export declare type DocEmbedOptions =
    | DocStaticEmbedOptions
    | DocEditorEmbedOptions;

export declare type DocFindOptions = {
    /**
     * `true` if the search UI should go to the next result. Only makes sense if
     * the search UI is already showing.
     * Mutually exclusive with `findPrevious` and `replace`.
     */
    findNext?: boolean;
    /**
     * `true` if the search UI should go to the previous result. Only makes
     * sense if the search UI is already showing.
     * Mutually exclusive with `findNext` and `replace`.
     */
    findPrevious?: boolean;
    /**
     * `true` to show the replace UI, not the find UI. Changes the search mode
     * if the UI is already showing.
     * Mutually exclusive with `findNext` and `findPrevious`.
     */
    replace?: boolean;
    /**
     * Specify a value to set the search text. Replaces the current search
     * text if the UI is already showing.
     */
    searchText?: string;
};

export declare enum DocInlineStyle {
    BOLD = "bold",
    ITALIC = "italic",
    UNDERLINE = "underline",
    STRIKETHROUGH = "strikethrough",
    CODE = "code",
    SUBSCRIPT = "subscript",
    SUPERSCRIPT = "superscript",
    LINK = "link",
}

export declare enum DocBlockStyle {
    PLAIN = "plain",
    H1 = "h1",
    H2 = "h2",
    H3 = "h3",
    CODE = "code",
    BLOCKQUOTE = "blockquote",
    LIST_BULLET = "list_bullet",
    LIST_NUMBERED = "list_numbered",
    LIST_CHECKLIST = "list_checklist",
}

export declare type OpenLinkOptions = {
    focusSectionId?: string;
    /**
     * @deprecated
     */
    newWindow?: boolean;
    button?: number;
    modifiers?: ShortcutModifier | number;
    /**
     * Name of editor frame link was opened in
     */
    frameName?: string;
    fromThreadId?: string;
    /**
     * Text label of the link
     */
    label?: string;
};

export declare type Link = {
    url: string;
    label?: string;
    emoji?: string;
    faviconUrl?: string;
};

/**
 * Template type to insert into doc
 */
export declare enum TemplateTriggerType {
    TODO_LISTS = "todo_lists",
    LINKS_N_RESOURCES = "links_and_resources",
    MEETING_NOTES = "meeting_notes",
    BLANK_PAGE = "blank_page",
}

/**
 * Options when inserting a template
 * replace entire doc content when replaceContent set to true
 * exampleUrl is used for links and unfurls from a template
 */
export declare type TemplateTriggerOptions = {
    replaceContent?: boolean;
    exampleUrl?: string;
    suppressFocus?: boolean;
};

/**
 * Scroll position to save and restore. Should be treated as an opaque type.
 */
export declare type DocScrollPosition = {
    readonly top: number;
};

/**
 * Info about the current focus state in the editor. Should be treated as an
 * opaque type and can be extended in the future if needed/desired.
 */
export declare type DocFocusState = {
    readonly sectionId: string;
};

/**
 * Info about loading the doc. If web caching is enabled and where
 * the doc was loaded from.
 */
export declare type DocLoadInfo = {
    readonly webCachingEnabled: boolean;
    readonly loadedFrom: ThreadLoadedFrom;
};

export declare enum ThreadLoadedFrom {
    CACHE = "cache",
    MEMORY = "memory",
    NETWORK = "network",
}

export declare type SelectedCommentState = {
    selectedCommentThreads: number;
    commentCount?: number;
};

export declare type TemplateControlsInfo = Map<
    string,
    {
        name: string;
        type: TemplateType;
        controlIds: Set<string>;
    }
>;

/**
 * Info about the current selection state of sections/styles in the editor; can
 * be used to determine how to render formatting/insertion toolbars. Should be
 * treated as an opaque type.
 */
export declare type DocSelectionState = {
    readonly caretRange: Range;
    readonly caretSectionId: string;
    readonly selectedSectionCount: number;
    readonly canEditStyles: boolean;
    readonly inlineStyles: DocInlineStyle[];
    readonly selectedCommentState: SelectedCommentState;
    readonly disallowedInsertionTypes?: Set<SectionInsertionType>;
    readonly disallowedShortcutTypes?: Set<SlackShortcutCommandType>;
    readonly blockStyle?: DocBlockStyle;
    readonly popoverOpen?: boolean;
    readonly commentsDisallowed?: boolean;
};

export declare enum OpenCommentScrollBehavior {
    NONE = "none",
    SCROLL_TO_SECTION = "scroll_to_section",
    FOCUS_SECTION = "focus_section",
}

/** Set of options passed while inserting a slack object */
export declare type InsertSlackObjectOptions = {
    x?: number;
    y?: number;
    sectionId?: string;
    isBookmarked?: boolean;
    /**
     * If set to true insert the object in the format most
     * appropriate given the current selection. Currently
     * that means that the object will be inserted as a card if
     * on an empty line and as title view otherwise.
     */
    useDefaultFormat?: boolean;
    /**
     * Objects inserted in a grid like layout when the value is set and met other conditions such as a targetSectionId is passed
     * as well as `isEligibleForGridlikePlacement` is true
     */
    shouldAddInGrid?: boolean;
};

declare type EmbeddedFile = {
    fileId: string;
    entityId: string;
};

export declare type GetEmbeddedFilesResponse = {
    files: EmbeddedFile[];
};

export declare enum ToastMessageContext {
    LINK_COPIED = 0,
    LINK_FILE_SHARE_BLOCKED = 1,
    TABLE_SIZE_LIMIT_REACHED = 2,
}

/**
 * Documents functionality provided by controller.
 */
export interface ControllerDocs {
    createDoc(
        title?: string,
        cannedDoc?: CreateDocumentOptions["cannedDoc"],
        context?: string,
        usesOnlyInlineTextComments?: boolean,
        content?: string
    ): Promise<string>;
    reloadThread(threadId: string): void;
    setDebugOption(key: string, value: boolean): void;
    findSlackObject(
        threadId: string,
        type: SlackObjectType,
        id?: string
    ): Promise<string | null>;
    insertSlackObject(
        docId: string,
        object: SlackObject,
        focusedSectionId?: string,
        url?: string,
        isBookmarked?: boolean
    ): Promise<string | null>;
    editLink(docId: string, controlId: string, link: Link): void;
    loadData(docIds: string[], reason: string): Promise<null>;
    preloadData(docIds: string[], reason: string): void;
    getDocEditor(frameName: string): DocEditor;
}

/**
 * Per document functionality provided by controller.
 */
export interface DocEditor
    extends CommentsUiProviderDelegate,
        AutocompleteUiProviderDelegate,
        EditorUiConnectorDelegate {
    insertLink(): void;
    insertSection(type: SectionInsertionType, source?: string): void;
    addComment(
        source?: string,
        skipOpenTab?: boolean
    ): Promise<string | undefined>;
    insertSlackObject(
        object: SlackObject | SlackObject[],
        options?: InsertSlackObjectOptions,
        source?: string
    ): void;
    insertSlackObjects(
        objects: SlackObject[],
        options?: InsertSlackObjectOptions,
        source?: string
    ): void;
    find(options?: DocFindOptions): void;
    executeFindQuery(query: string): void;
    executeReplaceQuery(replaceText: string, all?: boolean): void;
    findModalClosed(): void;
    moveListItem(moveUp: boolean): void;
    toggleInlineStyle(inlineStyle: DocInlineStyle, source?: string): void;
    canInsertInSection(sectionId: string): Promise<boolean>;
    setBlockStyle(blockStyle: DocBlockStyle, source?: string): void;
    getSelectedContent(): Promise<{
        [mimeType: string]: string;
    }>;
    getFullDocumentContent(): Promise<string>;
    toggleBlockStyle(blockStyle: DocBlockStyle, source?: string): void;
    cycleSectionStyle(forward?: boolean): void;
    undo(): void;
    redo(): void;
    showContextMenu(): void;
    focusEditor(): void;
    blurEditor(): void;
    focusSection(sectionId: string): void;
    applyFocusState(focusState: DocFocusState): void;
    insertContentFromTemplate(
        templateTriggerType: TemplateTriggerType,
        templateTriggerOptions?: TemplateTriggerOptions
    ): Promise<boolean>;
    insertContent(content: string): Promise<boolean>;
    openOrStartCommentsThread(source?: string): Promise<boolean>;
    copyAnchorLink(): Promise<boolean>;
    setCurrentlyOpenCommentId(
        commentId: string,
        behavior: OpenCommentScrollBehavior
    ): void;
    setEditsDisabled(disableEdits: boolean): void;
    getEmbeddedFiles(): Promise<GetEmbeddedFilesResponse>;
    getIsDirty(): Promise<boolean>;
    toggleA11yReadOnlyMode(): void;
    highlightSectionsEditedAfterTimestamp(
        timestamp: number,
        allowSameUser?: boolean,
        callback?: (
            sectionToReturn: {
                sectionId: string;
                author?: string | null;
                timestamp?: number | null;
            }[]
        ) => void
    ): void;
    unhighlightAllSections(): void;
    changeSectionHighlightBySectionId(
        sectionId: string,
        highlighted: boolean
    ): void;
    setVisibleBottomOffset(visibleBottomOffset: number): void;
    scrollToNextOffscreenHighlight(): void;
    setHeaderImage(
        image: SlackFile | SlackBackground | null,
        source?: string
    ): void;
    openHeaderImagePicker(): void;
}

export declare type ValueListener<T> = (
    oldValue: T | undefined,
    currentValue: T
) => void;

/**
 * Holder for dynamic values returned by API methods. Allows the current
 * value to be gotten, and listeners to be registered to allow the client
 * to be notified of changes.
 * Values may be `undefined` if data has not loaded yet.
 */
export interface Value<T> {
    get(): T | undefined;
    getAsync(): Promise<T | undefined>;
    listen(listener: ValueListener<T>): void;
    unlisten(listener: ValueListener<T>): void;
    once(listener: ValueListener<T>): void;
}

export declare enum AtomType {
    DOC_DISPLAY_TITLE = 0,
    DOC_UPDATED_USEC = 1,
    DOC_EMBED_SCROLL_POSITION = 2,
    DOC_BLOCK_KIT_THUMBNAIL = 3,
    SECTION_SNIPPET = 4,
    CANNED_DOC_NAME = 5,
    DOC_IS_EMPTY = 6,
    DOC_LAST_UPDATED_USER_ID = 8,
    A11Y_READ_ONLY_MODE = 9,
    DOC_EMBED_FOCUS_STATE = 10,
    DOC_EMBED_SELECTION_STATE = 11,
    TEXT_SECTION_OVERLAY_SHOWING = 12,
    DOC_HAS_FILE_UNFURLS = 13,
    TEMPLATE_CONTROLS_BE = 14,
    DOC_NEW_SECTION_USEC = 15,
    TEMPLATE_CONTROLS_FE = 16,
    DOC_HAS_HEADER_IMAGE = 17,
}

export interface AtomValueTypeMap {
    [AtomType.DOC_DISPLAY_TITLE]: string;
    [AtomType.DOC_UPDATED_USEC]: number;
    [AtomType.DOC_EMBED_SCROLL_POSITION]: DocScrollPosition;
    [AtomType.DOC_BLOCK_KIT_THUMBNAIL]: string;
    [AtomType.SECTION_SNIPPET]: string;
    [AtomType.CANNED_DOC_NAME]: string;
    [AtomType.DOC_IS_EMPTY]: boolean | null;
    [AtomType.DOC_LAST_UPDATED_USER_ID]: string;
    [AtomType.A11Y_READ_ONLY_MODE]: boolean;
    [AtomType.DOC_EMBED_FOCUS_STATE]: DocFocusState | null;
    [AtomType.DOC_EMBED_SELECTION_STATE]: DocSelectionState | null;
    [AtomType.TEXT_SECTION_OVERLAY_SHOWING]: boolean;
    [AtomType.DOC_HAS_FILE_UNFURLS]: boolean;
    [AtomType.TEMPLATE_CONTROLS_BE]: TemplateControlsInfo;
    [AtomType.TEMPLATE_CONTROLS_FE]: TemplateControlsInfo;
    [AtomType.DOC_NEW_SECTION_USEC]: number;
    [AtomType.DOC_HAS_HEADER_IMAGE]: boolean;
}

export interface AtomStorageProvider {
    get(key: string): any;
    set(key: string, value: any): void;
    delete(key: string): void;
}

export interface AtomClient {
    getValue<T extends keyof AtomValueTypeMap>(
        type: T,
        key: string
    ): AtomValueTypeMap[T] | undefined;
    getValueAsync<T extends keyof AtomValueTypeMap>(
        type: T,
        key: string
    ): Promise<AtomValueTypeMap[T] | undefined>;
    addListener<T extends keyof AtomValueTypeMap>(
        type: T,
        key: string,
        listener: ValueListener<AtomValueTypeMap[T]>
    ): void;
    removeListener<T extends keyof AtomValueTypeMap>(
        type: T,
        key: string,
        listener: ValueListener<AtomValueTypeMap[T]>
    ): void;
    cleanupStorage<T extends keyof AtomValueTypeMap>(
        type: T,
        key: string
    ): void;
}

export declare type InitError =
    | "no_jwt"
    | "malformed_jwt"
    | "no_quip_user"
    | "feature_disabled"
    | "exception";

export declare enum Feature {
    COMMENTS = 0,
    FILE_UPLOADS = 1,
    DOCUMENT_EVENTS = 2,
    SPREADSHEETS = 3,
    HOVERCARDS = 4,
    LINK_UNFURL = 5,
    UNREAD_COMMENTS = 6,
    REMINDERS = 7,
    DOCUMENT_UPDATE = 8,
    FLEXBOX_COLLAB = 9,
    SLACK_WORKFLOW_BUTTON_PROTOTYPE = 10,
    DOPE_TABLE = 11,
    UI_PARITY = 12,
    MOBILE_DOCUMENT_EDITING = 13,
    MOBILE_IMAGE_LOADING = 14,
    GALLERY_LAYOUT = 15,
    ANCHOR_LINKS = 16,
    IMAGE_EMBEDS = 17,
    IMAGE_CAPTION = 18,
    EMBEDS_DRAG_HANDLE = 19,
    SLACK_AUTOCOMPLETE = 20,
    DOCUMENT_SLASH_COMMANDS = 23,
    DRAG_AND_DROP_MAX_DISTANCE = 24,
    APP_UNFURL_NON_IFRAME_RENDERS = 25,
    A11Y_KEYBOARD_NAVIGATION = 28,
    INSERT_TOOLBAR = 29,
    MENTIONS_SHARE = 30,
    TEXT_IN_GALLERY_LAYOUT = 31,
    AUTOSIZE_FLEXBOX = 32,
    FORMATTING_TOOLBAR = 33,
    UI_PARITY_SLACK_LINK_INSPECTOR = 34,
    PREVENT_OBJECTS_IN_LISTS = 35,
    DISALLOW_SECTION_COMMENTS = 37,
    SECTION_TOOLBAR_ON_LEFT = 38,
    MOBILE_WEBVIEW_UNFURLS = 39,
    INLINE_COMMENTS = 40,
    UNIVERSAL_DRAG_PREVIEW = 41,
    DISABLE_GLOBAL_SHORTCUTS = 42,
    NARROW_COLLAB_INDENTATION = 43,
    EXPANDED_FLEXBOX_MARGIN = 44,
    ADD_COLUMN_FROM_SECTION_MENU = 45,
    REQUEST_COMPLETION_HOOK = 46,
    A11Y_CARET = 47,
    BOOKMARKS = 48,
    GRIDLIKE_DRAG_AND_DROP_PLACEMENT = 50,
    SLACK_MESSAGE_TITLE = 51,
    LENIENT_STARTUP = 52,
    ALLOW_SELECTING_ARCHIVED_COMMENTS = 53,
    ANIMATE_LAYOUT_DRAG = 54,
    STATIC_TITLES = 55,
    PROFILE_UNFURLS = 56,
    DISABLE_PERSISTING_CLIENT_USER_DOCUMENT_OVERLAY_UPDATES = 57,
    MOBILE_NATIVE_UNFURLS = 58,
    DEFAULT_UNFURLS_ON_PASTE = 59,
    SAVED_FOR_LATER = 60,
    COMMENTS_REDESIGN = 61,
    SLACK_USER_CONTEXT_MENU = 62,
    SLACK_OBJECT_CONTROL_CONTEXT_MENUS = 63,
    JS_MODULE_PREFETCH = 64,
    SLACK_SECTION_MENUS = 65,
    PADDING_RESIZING = 66,
    HOVERABLE_SECTION_MENU = 68,
    SLACK_PRESENCE_UPDATES = 69,
    HIDE_INLINE_COMMENT_BUBBLES = 70,
    HIDE_ADD_COMMENT_BUBBLE = 71,
    HYPERLINK_CONTEXT_MENU = 72,
    UNIVERSAL_DRAG_PREVIEW_EMBEDDED_UI = 73,
    LOCK_SECTION_FOR_INLINE_ANNOTATION = 74,
    EDITABLE_CONTROL = 75,
    OPEN_ANNOTATION_TAB_IN_FOCUSED_SECTION = 76,
    MOVE_CARET_WHEN_OPENING_ANNOTATION = 77,
    DISABLE_TITLE_UNFURL = 79,
    HIDE_STUB_ANNOTATIONS = 80,
    OBSERVE_CONTENT_OUTSIDE_SECTION_MUTATIONS = 81,
    USE_EXTERNAL_EDITOR_SAVE = 82,
    IMPROVED_WORD_WRAPPING = 83,
    HIGHLIGHT_TEXT_WITH_SECTION_LEVEL_COMMENTS = 84,
    USE_NEW_FIND_MODAL = 85,
    DISPLAY_COMMENT_TOOLTIP = 86,
    HIDE_SLACK_SECTION_HIGHLIGHT_FOR_OPEN_COMMENT = 87,
    SLACK_FILE_UNFURL_PLACEHOLDER = 88,
    PRIORITIZE_OPENING_ANNOTATIONS_OVER_LINKS = 89,
    CLEANUP_SENTINEL = 90,
    OPEN_COMMENTS_HIGHLIGHT_TEMPORARY = 91,
    UNFURL_PRIVATE_MESSAGES = 92,
    MOBILE_DISABLE_PRESENCE_MENU_POPOVER = 93,
    CANVAS_GA_FAST_FOLLOW = 95,
    BRIDGE_REFACTORING = 96,
    SIMPLE_TABLES = 97,
    ALLOW_LARGE_SIZE_FILE_IMAGE_UPLOADS = 98,
    ALLOW_SLACK_OBJECT_TITLE_UNFURL = 99,
    ALLOW_SLACK_WEB_OR_APP_LINKS_TITLE_UNFURL = 100,
    CANVAS_GA_FAST_FOLLOW_2 = 101,
    USE_LEFT_GUTTER_DISPLAY = 102,
    USE_PRIVATE_MESSAGE_SPEEDBUMP_IN_CARD = 103,
    TITLE_BLOCK_KIT_FORMAT = 104,
    USE_LIST_ITEM_MARKER_CONTAINER = 105,
    ALLOW_SLACK_TEMPLATE_PLACEHOLDER = 106,
    USE_INLINE_TOOLBAR_PERF_IMPROVEMENTS = 107,
    IMAGE_RESIZING = 108,
    PRIVATE_MESSAGE_SPEEDBUMP_MODAL = 109,
    ALLOW_HEADER_IMAGE = 110,
    PRIVATE_MESSAGE_SPEEDBUMP_FOR_TITLE_UNFURLS = 111,
    USE_NEW_SECTION_HIGHLIGHT = 112,
    MODEL_INTEGRITY_PROBE = 113,
    MODEL_INTEGRITY_CHANGES_CHECK = 114,
    MODEL_FSCK_PROBE = 115,
    MODEL_UPDATES_PROBE = 116,
    SIMPLE_TABLES_FULL_BLEED = 117,
    IMAGE_MIN_DIMENSIONS = 118,
    SIMPLE_TABLES_MOBILE_DISPLAY_SCALE = 119,
    SLACK_TEXTY_CLIPBOARD = 120,
    ONLY_OPEN_ANNOTATION_WHEN_CARET_IS_INSIDE = 121,
    SINGLE_HOVERABLE_SECTION_MENU = 122,
    SIMPLE_TABLES_FIT_TO_PAGE = 123,
    TRANSIENT_SECTION_HIGHLIGHTING = 124,
    OPTIMIZE_CONTROLS_RENDERING = 125,
    SIMPLE_TABLES_MOBILE_EDITING = 126,
    OPTIMIZE_EMBEDS_RENDERING = 127,
    CONTENT_REACTION_BAR = 128,
    GUTTER_REDESIGN = 129,
    SLASH_SHORTCUT_COMMANDS = 130,
    USE_NATIVE_LINK_INSPECTOR = 131,
    SECTION_UPDATE_PILL_ENABLED = 132,
}

export interface ComponentRef {
    /**
     * Remove the component from the container where it was mounted. Client
     * is responsible for doing this (even if the container for the
     * component has been removed from the document DOM) to ensure that
     * all resources related to this component are released.
     */
    unmount(): void;
    /**
     * Returns a reference to the corresponding container for this component
     */
    containerElement(): Element;
    /** Allows client to know when the component has been fully rendered. */
    readonly rendered: Value<boolean>;
    /** Allows client to know when the component has been fully rendered with
     * all the incremental renders. */
    readonly incrementalRendered: Value<boolean>;
    /** Allows client to know when the component has failed to load. */
    readonly failedToLoad: Value<boolean>;
    /** Contains the error message in cases where the embed failed to load. */
    readonly loadError: Value<string | undefined>;
    /** Contains info about loading the thread */
    readonly loadInfo: Value<DocLoadInfo | undefined>;
    /**
     * DIRECT_RENDER_ELEMENT only: Allows client to listen for when the editor
     * container is created on document rendering.
     */
    readonly editorContainer: Value<HTMLElement | undefined>;
    /** Allows the client to know if an editing event has been performed
     */
    readonly editingActionPerformed: Value<boolean>;
}

export declare enum CollabUrlType {
    SLACK_MESSAGE = 0,
    SLACK_FILE = 1,
    SLACK_IMAGE_FILE = 8,
    SLACK_DOCUMENT = 2,
    SLACK_DOCUMENT_SECTION = 3,
    SLACK_PUBLIC_LINK = 4,
    SLACK_USER = 5,
    SLACK_CHANNEL = 6,
    SLACK_WORKFLOW = 7,
}

export declare enum UnfurlType {
    CARD = 0,
    INLINE = 1,
    LINK = 2,
}

export declare type UrlTypeToRegExp = {
    urlType: CollabUrlType;
    regExp: RegExp;
};

export declare enum SlackObjectActionType {
    DOWNLOAD = "download",
    EDIT_FILE = "edit-file",
    OPEN_IN_NEW_WINDOW = "open-in-new-window",
    PREVIEW_FILE = "preview-file",
    VIEW_FULL_SCREEN = "view-full-screen",
    SHARE = "share",
}

export declare type PerformSlackObjectAction = {
    objectPermalink: string;
    action: SlackObjectActionType;
    objectType: SlackObjectType;
    quipThreadId?: string;
    unfurlType?: UnfurlType;
};

/**
 * Definition of metrics that are recoded and sent to the host (name and known
 * arguments). Done as an actual value (as opposed to just a type) so that we
 * can also use it for filtering in the implementation.
 */
export declare const RecordedMetrics: {
    /** Editing */
    readonly select_style: {
        readonly select_style_type: 0;
        readonly select_style_source: 0;
        readonly in_table: 0;
    };
    readonly toggle_inline_style: {
        readonly style: 0;
        readonly source: 0;
    };
    readonly change_checkbox: {
        readonly checked: 0;
    };
    readonly change_indentation_level: {
        readonly indent: 0;
        readonly new_level: 0;
        readonly old_level: 0;
    };
    readonly change_text_alignment: {
        readonly alignment: 0;
        readonly source: 0;
    };
    readonly edit_layout: {
        readonly action: 0;
        readonly in_table: 0;
    };
    readonly first_edit_document_after_open: {};
    readonly edit_document: {};
    readonly clipboard_event: {
        readonly event_type: 0;
        readonly data_transfer_item_count: 0;
    };
    /** Menus */
    readonly menu_handle_command_document: {
        readonly command: 0;
        readonly subcommand: 0;
    };
    readonly menu_handle_command_section_context: {
        readonly command: 0;
        readonly subcommand: 0;
    };
    readonly menu_handle_command_document_settings: {
        readonly command: 0;
        readonly subcommand: 0;
    };
    /** List and heading collapsing */
    readonly toggle_list_item_collapsed_state: {
        readonly source: 0;
    };
    readonly toggle_section_collapsed_state: {
        readonly source: 0;
    };
    readonly toggle_sublists_collapsed_state: {
        readonly source: 0;
    };
    readonly toggle_all_collapsed_state: {
        readonly source: 0;
    };
    /** Insert menu */
    readonly open_rich_insert_menu: {
        readonly source: 0;
    };
    readonly insert_menu_select: {
        readonly action: 0;
        readonly source: 0;
    };
    readonly insert_figure: {
        readonly figure_type: 0;
        readonly insert_figure_source: 0;
        readonly file_type: 0;
        readonly url_type: 0;
        readonly in_table: 0;
    };
    /** Search (i.e. in-document find) */
    readonly nav_search_show: {
        readonly source: 0;
    };
    readonly nav_search_hide: {
        readonly source: 0;
    };
    /** Link inspector */
    readonly link_inspector_create_link: {
        readonly source: 0;
    };
    readonly link_inspector_edit_link: {
        readonly source: 0;
    };
    readonly link_inspector_copy_link: {
        readonly source: 0;
    };
    readonly link_inspector_remove_link: {
        readonly source: 0;
    };
    readonly link_indicator_shown: {};
    /** Unprompted model probes */
    readonly unprompted_model_probe: {};
    readonly unprompted_model_probe_changes_found: {
        readonly updated_section_count: 0;
        readonly updated_document_count: 0;
        readonly updated_thread_count: 0;
        readonly duration_since_server_update_ms: 0;
        readonly in_flexpane: 0;
        readonly edits_disabled: 0;
        readonly edits_transiently_disabled: 0;
        readonly old_sequence_num: 0;
        readonly new_sequence_num: 0;
    };
    readonly model_checksum_attempted: {};
    readonly model_checksum_mismatch: {
        readonly client_checksum: 0;
        readonly server_checksum: 0;
    };
    readonly document_fsck_probe: {};
    readonly document_fsck_probe_failure: {
        readonly summary: 0;
    };
    /** Unfurls */
    readonly link_menu_command_click: {
        readonly command_id: 0;
        readonly source: 0;
        readonly url_type: 0;
        readonly unfurl_type: 0;
    };
    readonly link_unfurl_success: {
        readonly url_type: 0;
        readonly unfurl_type: 0;
        readonly slack_object_type: 0;
        readonly hostname: 0;
        readonly app_id: 0;
        readonly state: 0;
    };
    readonly link_unfurl_failure: {
        readonly url_type: 0;
        readonly unfurl_type: 0;
        readonly failure_reason: 0;
        readonly hostname: 0;
        readonly app_id: 0;
    };
    readonly unfurl_convert: {
        readonly slack_object_type: 0;
        readonly from_type: 0;
        readonly to_type: 0;
    };
    /** Comments */
    readonly open_comment: {
        readonly source: 0;
        readonly message_count: 0;
        readonly is_section_highlight: 0;
        readonly section_type: 0;
    };
    readonly start_comments_thread: {
        readonly mapped_authors_count: 0;
        readonly unmapped_authors_count: 0;
    };
    /** General click tracking */
    readonly click: {
        readonly click_name: 0;
    };
    /** Templates */
    readonly template_trigger_clicked: {
        readonly template_type: 0;
    };
    readonly template_help_center_link_clicked: {};
    /**Image resizing */
    readonly image_resize_fixed_width: {
        readonly layout_mode: 0;
        readonly fixed_width: 0;
        readonly fixed_width_old: 0;
        readonly resize_handle: 0;
    };
    /** Table edits */
    readonly table_resize_column: {
        readonly table_id: 0;
        readonly index: 0;
        readonly old_width: 0;
        readonly new_width: 0;
    };
    readonly table_add_rows: {
        readonly table_id: 0;
        readonly count: 0;
        readonly start_index: 0;
    };
    readonly table_add_columns: {
        readonly table_id: 0;
        readonly count: 0;
        readonly start_index: 0;
    };
    readonly table_remove_rows: {
        readonly table_id: 0;
        readonly count: 0;
        readonly start_index: 0;
    };
    readonly table_remove_columns: {
        readonly table_id: 0;
        readonly count: 0;
        readonly start_index: 0;
    };
    readonly table_move_row: {
        readonly table_id: 0;
        readonly new_index: 0;
        readonly original_index: 0;
    };
    readonly table_move_column: {
        readonly table_id: 0;
        readonly new_index: 0;
        readonly original_index: 0;
    };
    readonly table_fit_to_page: {
        readonly enabled: 0;
        readonly table_width: 0;
    };
};

export declare const CommonMetricsArgs: {
    readonly thread_id: 0;
};

/**
 * A description of a completed request.
 */
export declare type RequestCompletion = {
    method: string;
    url: string;
    requestHeaders: {
        [key: string]: string | string[];
    };
    responseHeaders: {
        [key: string]: string | string[];
    };
    requestBody?:
        | string
        | {
              [key: string]: string;
          };
    responseBody?: Blob;
};

export declare type Permalink = {
    objectPermalink: string;
};

export interface SlackObjectUIProvider {
    saveSlackObject({objectPermalink}: Permalink): void;
    removeSavedSlackObject({objectPermalink}: Permalink): void;
    performSlackObjectAction({
        objectPermalink,
        action,
        objectType,
        quipThreadId,
        unfurlType,
    }: PerformSlackObjectAction): void;
}

export interface PlatformDelegate {
    /**
     * Successfully created the initial (authenticated) connection to the Quip
     * server. SDK method calls will be buffered until this is complete.
     */
    didInitialize?(): void;
    /**
     * If collab.init fails to load this will be called with one of the errors
     * defined in InitError.
     */
    didFailToInitialize?(initError: InitError, err?: Error): void;
    /**
     * Lifecycle events for the listen (real-time updates) server.
     * Connects and disconnects may happen normally due to users going
     * idle/becoming active, network changes, etc.
     */
    didConnectToListenServer?(): void;
    didDisconnectFromListenServer?(): void;
    /**
     * Invoked for cases where a connection is never established (because of
     * auth, networking, etc.), and realtime-related features may not work at
     * all (the host may wish to hide or disable them).
     */
    didFailToConnectToListenServer?(): void;
    editLink?(docId: string, controlId: string, link: Link): boolean;
    /**
     * The user indicated that they want to open an external link (e.g.
     * one embedded in a doc), possibly in a new window. If the host cannot
     * perform this operation, false should be returned.
     * @deprecated use PermalinkUiProvider
     */
    openLink?(url: string, options?: OpenLinkOptions): boolean;
    /**
     * The user indicated that they wish to navigate to a document
     * (e.g. because of clicking on a mention). If the host cannot perform
     * this navigation, false is returned.
     * @deprecated use PermalinkUiProvider
     */
    openDoc?(id: string, options?: OpenLinkOptions): boolean;
    /**
     * Similar open hooks for Slack entities
     * @deprecated use PermalinkUiProvider
     */
    openSlackChannel?(channelId: string, options?: OpenLinkOptions): boolean;
    /** @deprecated use PermalinkUiProvider */
    openSlackUser?(userId: string, options?: OpenLinkOptions): boolean;
    /**
     * Called when the Quip limited session has expired and a new JWT is
     * required to exchange for a new session.
     */
    refreshUserJwt?(): Promise<string | undefined>;
    /** Called when a mounted component gains focus. */
    componentDidFocus?(component: ComponentRef): void;
    /** Called when a mounted component loses focus. */
    componentDidBlur?(component: ComponentRef): void;
    /** @deprecated use SlackObjectUIProvider instead. */
    saveSlackObject?({objectPermalink}: Permalink): void;
    /** @deprecated use SlackObjectUIProvider instead. */
    removeSavedSlackObject?({objectPermalink}: Permalink): void;
    /** @deprecated use SlackObjectUIProvider instead. */
    performSlackObjectAction?({
        objectPermalink,
        action,
        objectType,
    }: PerformSlackObjectAction): void;
    /**
     * Called when a user action took place and metrics data was generated for
     * it.
     */
    didRecordMetric?<M extends keyof typeof RecordedMetrics>(
        metric: M,
        args: {
            [Arg in keyof (
                | typeof RecordedMetrics[M]
                | typeof CommonMetricsArgs
            )]: string;
        }
    ): void;
    /**
     * Called when specific client performance metrics are finished and metrics
     * data was generated for it.
     */
    didRecordClientperfMetric?(
        metric: string,
        startTime: number,
        endTime: number,
        args: {
            [key: string]: string;
        }
    ): void;
    /**
     * The controller completed a network request.  This hook is suitable for
     * recording/logging that request, but does not permit it to be prevented
     * or modified.  Only gets called if the REQUEST_COMPLETION_HOOK feature is
     * enabled.
     */
    didCompleteNetworkRequest?(completion: RequestCompletion): void;
    /**
     * Recorded a diagnostic event
     */
    didRecordDiagnosticsEvent?(
        event: {
            [key: string]: string;
        },
        isFrequent: boolean,
        isInvariantFailure: boolean
    ): void;
    /**
     * Called when a document is saved -- can be used to power a saved X
     * minutes ago indicator
     *
     * @param docId - document thread ID
     */
    didSaveDoc?(docId: string): void;
    updateTitle?(
        newTitleString: string,
        docId: string,
        isEmpty: boolean,
        newTitleBlocks?: SlackRichTextApi[]
    ): void;
    updatePresenceInfo?(editors: string[], threadId: string): void;
    openFindModal?(currentSelection?: string, replace?: boolean): void;
    updateFindResults?(resultCount: number, currentResultIndex: number): void;
    showToastForContext?(
        context: ToastMessageContext,
        targetWindow: Window
    ): void;
    /**
     * Start a trace, which is necessary before spans can be created.  Will
     * return null if tracing is not supported right now, or similar.  This
     * method should never throw.
     */
    startTrace?(
        traceName: string,
        tags?: {
            [key: string]: string;
        }
    ): Symbol | null;
    /**
     * Start a span.  The parentId may refer to either a span, or a trace. If
     * a null is passed as the parent id, this is a noop, and will return a
     * null.  A null will also be returned if tracing is currently disabled,
     * or something else goes wrong.  This method should never throw.
     */
    startSpan?(
        parentId: Symbol | null,
        spanName: string,
        tags?: {
            [key: string]: string;
        }
    ): Symbol | null;
    /**
     * Close out a trace or a span, this method will accept an id for either.
     * A noop if passed a null as the id.
     */
    endTraceOrSpan?(
        id: Symbol | null,
        tags?: {
            [key: string]: string;
        }
    ): void;
    a11yAnnounce?(
        window: Window,
        threadId: string,
        message: string,
        assertive?: boolean
    ): void;
    isOnline?(): boolean | undefined;
    updateFileReadTsBeforeSave?(threadId: string): void;
}

export declare type Snippet = {
    mrkdwn: string;
    sectionUsers: SectionUser[];
    fileIds?: string[];
    isArchived?: boolean;
    lastEditedUsec?: number;
    objectStubs?: SlackObjectStub[];
    createdUsec?: number;
    richText?: string;
    controlId?: string;
};

export declare type SectionUser = {
    slackUserId?: string;
    slackTeamId?: string;
    quipUserId?: string;
    isAuthor: boolean;
    isRecentEditor: boolean;
    isMostRecentEditor: boolean;
    isMentioned: boolean;
};

/**
 * Quip -> host API, to allow the host to create comments thread and other
 * data-related operations.
 */
export interface CommentsDataProvider {
    /**
     * Opens a comment thread. `commentThreadId` is a stable Quip-provided
     * identifier that can be used to reference the thread later.
     * * `snippet` is an object whose mrkdwn property is a Markdown
     * snippet of the content that the comment is associated with.
     */
    startCommentsThread(
        docId: string,
        commentThreadId: string,
        snippet: Snippet
    ): void;
    /**
     * Updates a comments thread. Used to keep the Slack comment root
     * in sync with the underlying section in the Quip doc
     */
    updateCommentsSnippet(
        docId: string,
        commentThreadId: string,
        snippet: Snippet | undefined,
        highlightUpdatedTimestamp: number,
        isSourceDeleted: boolean
    ): void;
    setDelegate(delegate: CommentsDataProviderDelegate): void;
}

/**
 * Host -> Quip Comments API, for informing the Quip client side of comments
 * changes that happened on the host side. These are optimistic updates to
 * reduce user-perceived latency, the actual updates happen via the private
 * Slack -> API server-side API.
 */
export interface CommentsDataProviderDelegate {
    /**
     * Messages were added/removed from the comments thread (the Quip doc
     * contains a denormalized map of comment counts to display in the gutter
     * immediately at load time and also update the latest message sequence
     * which updates the read/ unread state).
     */
    commentsThreadMessageDidChange(
        docId: string,
        commentThreadId: string,
        param: {
            messageCount?: number;
            latestMessageSequence?: number;
        }
    ): void;
    /**
     * The entire comment thread was archived, the comments bubble should be
     * hidden.
     */
    commentsThreadArchiveStateChanged(
        docId: string,
        commentThreadId: string,
        threadArchived: boolean
    ): void;
    /**
     * The entire comment thread was deleted, the comments bubble should be
     * removed.
     */
    commentsThreadWasDeleted(docId: string, commentThreadId: string): void;
    /**
     * Gets a Markdown snippet for the document content that the given
     * section is about.
     *
     * Client-side alternative to the `GET /v1/comments/<id>/snippet` server
     * API, depending on which is more convenient.
     */
    getCommentsSnippet(
        docId: string,
        commentId: string
    ): Promise<Snippet | undefined>;
    /**
     * Updates the user comment fields
     */
    commentsThreadUserStateChanged(
        docId: string,
        commentThreadId: string,
        params: {
            userMarkUnread: boolean;
            lastSeenMessageSequence: number;
        }
    ): void;
    /**
     * Archive all the comments in a document
     */
    commentsArchiveAll(docId: string): void;
    /**
     * Get the last seen state for a comment in a given doc
     */
    commentsGetLastSeen(
        docId: string,
        commentThreadId: string
    ): Promise<number | null>;
    /**
     * Check whether a given comment thread ID is orphaned in the given doc.
     */
    commentsCheckIsOrphaned(
        docId: string,
        commentThreadId: string
    ): Promise<boolean>;
}

/**
 * Quip -> host API to allow us to react to sections being bookmarked/unbookmarked
 */
export interface BookmarksDataProvider {
    sectionIsBookmarkedChanged(props: {
        docId: string;
        sectionId: string;
        isBookmarked: boolean;
        slackObject?: SlackObjectStub;
    }): void;
    setDelegate(delegate: BookmarksDataProviderDelegate): void;
}

/**
 * Host -> Quip API for informing when bookmarks have been removed
 */
export interface BookmarksDataProviderDelegate {
    sectionIsBookmarkedChanged(props: {
        docId: string;
        sectionId: string;
        isBookmarked: boolean;
    }): Promise<boolean | null>;
}

export declare enum SlackImageSizeName {
    ORIGINAL = "original",
    THUMB_64 = "thumb_64",
    THUMB_80 = "thumb_80",
    THUMB_160 = "thumb_160",
    THUMB_360 = "thumb_360",
    THUMB_480 = "thumb_480",
    THUMB_720 = "thumb_720",
    THUMB_800 = "thumb_800",
    THUMB_960 = "thumb_960",
    THUMB_1024 = "thumb_1024",
    THUMB_360_GIF = "thumb_360_gif",
    DEANIMATED = "deanimate",
    DEANIMATED_GIF = "deanimate_gif",
}

export declare type ImageSize = {
    sizeName: SlackImageSizeName;
    width: number;
    height: number;
};

export declare type UploadFileResult = {
    name: string;
    size: number;
    mimeType: string;
    slackFileId: string;
    sizes: ImageSize[];
    deleted: boolean;
    altText: string;
    width: number;
    height: number;
    id: string;
    url: string;
    permalink: string;
};

export declare type UploadFilesResponse = {
    results: UploadFileResult[];
};

export interface FileDataProvider {
    /**
     * Uploads a list of files and returns metadata for each file.
     */
    uploadFiles(
        files: File[],
        showBanner?: boolean
    ): Promise<UploadFilesResponse>;
    processAndDecorateFileForUpload(file: File): File;
}

/**
 * Host -> Quip API: Host informs Collab SDK of updates to Slack objects.
 */
export interface SlackObjectProviderDelegate {
    /**
     * Notify the Collab SDK of Slack objects that have been updated. Any
     * objects that are of interest to the SDK will get the SDK to request the
     * updated states of these objects through PlatformDelegate `getSlackObjects`.
     */
    slackObjectsUpdated(objectsBase: SlackObjectStub[]): void;
    receivedSlackObjects(objects: SlackObject[]): void;
}

/**
 * Quip -> host API: Host provides Slack-related data to power features such as
 * mentions and autocomplete
 */
export interface SlackObjectProvider {
    /**
     * A list of autocomplete types for which the SlackObjectProvider should be
     * responsible for returning autocomplete results.
     */
    supportedAutocompleteTypes: SlackObjectType[];
    /**
     * A request for autocomplete results for the given type(s).
     * Results can be returned multiple times (if async requests are needed).
     * The document that the request is being made from is included in case
     * the context is useful.
     */
    getAutocompleteResults(
        query: string,
        autocompleteTypes: SlackObjectType[],
        docId?: string
    ): Promise<AutocompleteResponse>;
    /** A request for data about SlackObjects. */
    getSlackObjects(
        objects: SlackObjectStub[],
        incremental?: boolean
    ): Promise<GetSlackObjectsResponse>;
    /** A request for data about SlackObject by url. */
    getSlackObjectByUrl?(
        url: string,
        threadId: string,
        acknowledgedPrivateShare?: boolean
    ): Promise<GetSlackObjectsResponse>;
    createSlackCanvas?(
        parentThreadId?: string
    ): Promise<CreateSlackCanvasResponse>;
    getSlackEmojiByUnicodeString(
        unicode: string
    ): Promise<GetSlackEmojiByUnicodeStringResponse>;
    /** A request to insert a Slack object. */
    addSlackObject?(
        docId: string,
        objectType: SlackObjectType,
        focusedSectionId?: string
    ): void;
    /**
     * Collab SDK will set a SlackObjectProviderDelegate that the host can call
     * to inform the Collab SDK of updates.
     */
    setDelegate(delegate: SlackObjectProviderDelegate): void;
}

export declare type DocumentEventType = "transient" | "update" | "backlog";

/** Quip -> host API, to send document data event. */
export interface DocumentEventProvider {
    /**
     * Send document data updates.
     */
    sendDocumentEvent(
        docId: string,
        payload: Uint8Array,
        type: DocumentEventType
    ): void;
    setDelegate(delegate: DocumentEventProviderDelegate): void;
}

/**
 * Host -> Quip API, for unpacking document data event.
 */
export interface DocumentEventProviderDelegate {
    /**
     * Receive and apply document data updates.
     */
    receiveDocumentEvent(
        docId: string,
        payload: Uint8Array | string,
        type: DocumentEventType
    ): void;
    /**
     * Update the controller with authoritative presence information from
     * Slack's presence tracking system.  At the time of writing, this was
     * used mainly to help the controller decide how aggressively to deliver
     * updates to the server.  It doesn't power the presence UIs either in the
     * title bar or in the document body.
     */
    updatePresence(
        documents: {
            threadId: string;
            presenceUsers: {
                slackUserId: string;
                isWriting: boolean;
            }[];
        }[]
    ): void;
}

/** Quip -> host API, to send anchor link data. */
export interface AnchorLinkProvider {
    /** A request for getting a section anchor link */
    getSectionAnchorLink(
        docId: string,
        sectionId: string
    ): Promise<GetSectionAnchorLinkResponse>;
}

export interface SlackTextyHtmlConversionProvider {
    getHtmlFromSlackTextyJson(
        clipboardJsonString: string
    ): Promise<GetSlackTextyHtmlResponse>;
}

/**
 * Enum used for different error messages by source. The keys are in order of
 * importance.
 */
export declare enum ErrorSource {
    DOCUMENT_SAVING = 1,
    IO = 2,
    LISTENER = 3,
    ELEMENT = 4,
}

/**
 * Quip -> Slack API: Quip notifies Slack of entering and exiting of error
 * states a user should know of.
 */
export interface ErrorUiProvider {
    /**
     * Collab instance has entered into an error state that remains until it's
     * cleared by didClearErrorState.
     */
    didSetErrorState(
        source: ErrorSource,
        message: string,
        threadId: string,
        handlesOnClick: boolean,
        expiresAtMs: number | undefined
    ): void;
    /**
     * Collab instance is no longer in the error state for the given
     * ErrorSource.
     */
    didClearErrorState(source: ErrorSource): void;
    setDelegate(callbackDelegate: ErrorUiProviderDelegate): void;
}

/**
 * Slack -> Quip API to notify Quip the user has done some action with the error
 * notification.
 */
export interface ErrorUiProviderDelegate {
    /**
     * User has clicked on the error notification. Upon click, the Quip side
     * might clear the error state (for transient errors), force a save (for
     * some document saving errors), or do nothing.
     */
    onErrorClick(source: ErrorSource): void;
}

export declare enum EmbeddedUiContext {
    HOVERCARD = 0,
    CARD = 1,
    HEADER = 2,
    CONTROL_CONTEXT_MENU = 3,
    SECTION_MENU = 4,
    HYPERLINK_CONTEXT_MENU = 5,
    HYPERLINK_INSPECTOR_TOOLBAR = 6,
    CONTROL_OBJECT_OBSERVER = 7,
    TABLE_SELECTION_MENU = 8,
    FILE_UPLOAD_BLOCKED_OVERLAY = 9,
    EPHEMERAL_MESSAGE_MODAL_TRIGGER = 10,
    VARIABLE_EDIT_POPOVER = 11,
    CONTENT_REACTION_BAR = 12,
    GUTTER_REACTION_ICON = 13,
    HEADER_IMAGE_PICKER = 14,
}

export declare enum CollabPopoverDismissSource {
    ANY = 1,
    ON_MOUSE_LEAVE = 2,
}

export declare enum EmbeddedObjectType {
    SLACK_OBJECT = 0,
    LINK = 1,
    HEADER = 2,
    TEXT = 3,
    DIVIDER = 4,
    HYPERLINK = 5,
    TITLE = 6,
    TABLE = 7,
    DRAG_OVERLAY = 8,
    COMMENT_METADATA = 9,
}

export declare enum CardRenderType {
    DEFAULT = 0,
    BUTTON = 1,
}

export declare type EmbeddedObjectMetadata = {
    name: string;
    type?: string;
    description?: string;
    isUnfurled?: boolean;
};

export declare enum EmbeddedMoveDirection {
    PREVIOUS = 0,
    NEXT = 1,
}

export declare enum TablePart {
    ROW = "row",
    COLUMN = "column",
}

export declare enum TablePartModificationType {
    INSERT_BEFORE = "insert_before",
    INSERT_AFTER = "insert_after",
    DELETE = "delete",
}

export declare type TablePartModification = {
    part: TablePart;
    type: TablePartModificationType;
};

export declare enum TableSelectionModification {
    CLEAR_CONTENTS = "clear_contents",
}

export declare enum FullTableModification {
    TOGGLE_HAS_HEADER_ROW = "toggle_has_header_row",
    TOGGLE_HAS_HEADER_COLUMN = "toggle_has_header_column",
    DISTRIBUTE_COLUMNS = "distribute_columns",
    TOGGLE_FIT_TO_PAGE = "toggle_fit_to_page",
}

export declare type TableModification =
    | TablePartModification
    | TableSelectionModification
    | FullTableModification;

export declare type EmbeddedObject =
    | {
          type: EmbeddedObjectType.SLACK_OBJECT;
          object: SlackObjectStub;
      }
    | {
          type: EmbeddedObjectType.LINK;
          url: string;
      }
    | {
          type: EmbeddedObjectType.HEADER;
          object?: SlackObjectStub;
      }
    | {
          type: EmbeddedObjectType.TEXT;
      }
    | {
          type: EmbeddedObjectType.DIVIDER;
      }
    | {
          type: EmbeddedObjectType.TITLE;
      }
    | {
          type: EmbeddedObjectType.HYPERLINK;
          anchorElement: HTMLAnchorElement;
          inTitleSection: boolean;
          isEditing?: boolean;
      }
    | {
          type: EmbeddedObjectType.TABLE;
      }
    | {
          type: EmbeddedObjectType.DRAG_OVERLAY;
      }
    | {
          type: EmbeddedObjectType.COMMENT_METADATA;
          commentId: string;
      };

export declare type EmbeddedUiProviderConfig = {
    renderTargetUrl: string;
    supportedEmbededObjectTypes: EmbeddedObjectType[];
    supportedSlackObjectTypes: SlackObjectType[];
};

export declare enum EmbeddedUIAttributes {
    NO_FOCUS_ON_CLICK = "data-no-focus-section-on-click",
    NO_DRAG_ON_MOUSE_MOVE = "data-no-drag-on-mouse-move",
    EMBEDDED_FILE_CARD_CLASSNAME = "card-view__file_card",
    EMBEDDED_IMAGE_CARD_CLASSNAME = "card-view__image_card",
}

export declare enum EmbeddedUiAction {
    DISMISS_POPOVER = 0,
    REMOVE_ASSOCIATED_CONTENT = 1,
    COPY_OBJECT = 2,
    CREATE_OR_VIEW_COMMENT = 3,
    REPLACE_FILE = 4,
    SWITCH_UNFURL_TYPE = 5,
    PREVENT_POPOVER_DISMISS = 6,
    SYNC_METADATA = 7,
    COPY_SECTION_LINK = 8,
    SET_BLOCK_STYLE = 9,
    ADD_COLUMN = 10,
    MOVE_LINE = 11,
    TOGGLE_COLLAPSE = 12,
    CHECK_ALL = 13,
    UNCHECK_ALL = 14,
    REMOVE_CHECKED = 15,
    MOVE_CHECKED_TO_BOTTOM = 16,
    SORT_ALPHA = 17,
    TAKE_OVER_EDITING = 18,
    ADD_TEXT_BELOW = 19,
    UPDATE_INLINE_ENTITY_DISPLAY_INFO = 20,
    ON_LINK_INSPECTOR_INPUT_CHANGE = 21,
    FOCUS_ASSOCIATED_SECTION = 22,
    UNFURL_URL = 23,
    MODIFY_TABLE = 24,
    RESUME_SNAPSHOTS = 25,
    OPEN_POPOVER = 26,
}

export declare type InlineEntityDisplayInfo =
    | {
          context: Exclude<
              EmbeddedUiContext,
              EmbeddedUiContext.VARIABLE_EDIT_POPOVER
          >;
          displayLabel?: string;
          leadingEmoji?: string;
          faviconUrl?: string;
          url?: string;
      }
    | {
          context: EmbeddedUiContext.VARIABLE_EDIT_POPOVER;
          name?: string;
          type?: TemplateType;
      };

declare type TableSelectionOptionsGroup = {
    insertionEnabled: boolean;
    deletionEnabled: boolean;
    numSelected: number;
    toggleHeaderEnabled: boolean;
};

export declare type TableSelectionOptions = {
    rowOptions: TableSelectionOptionsGroup | null;
    columnOptions: TableSelectionOptionsGroup | null;
};

export declare type TableSectionAttributes = {
    hasHeaderRow: boolean;
    hasHeaderColumn: boolean;
    columnCount: number;
    columnsDistributed: boolean;
    viewOptions: TableViewOptions;
};

export declare type TableViewOptions = {
    isFitToPage: boolean;
    allowToggleFitToPage: boolean;
};

export declare type EmbeddedUiActionArgs =
    | {
          action: Exclude<
              EmbeddedUiAction,
              | EmbeddedUiAction.SWITCH_UNFURL_TYPE
              | EmbeddedUiAction.PREVENT_POPOVER_DISMISS
              | EmbeddedUiAction.SYNC_METADATA
              | EmbeddedUiAction.SET_BLOCK_STYLE
              | EmbeddedUiAction.ADD_COLUMN
              | EmbeddedUiAction.MOVE_LINE
              | EmbeddedUiAction.SORT_ALPHA
              | EmbeddedUiAction.TAKE_OVER_EDITING
              | EmbeddedUiAction.UPDATE_INLINE_ENTITY_DISPLAY_INFO
              | EmbeddedUiAction.COPY_SECTION_LINK
              | EmbeddedUiAction.ON_LINK_INSPECTOR_INPUT_CHANGE
              | EmbeddedUiAction.UNFURL_URL
              | EmbeddedUiAction.MODIFY_TABLE
          >;
      }
    | {
          action: EmbeddedUiAction.SWITCH_UNFURL_TYPE;
          toUnfurlType: UnfurlType;
      }
    | {
          action: EmbeddedUiAction.PREVENT_POPOVER_DISMISS;
          source?: CollabPopoverDismissSource;
      }
    | {
          action: EmbeddedUiAction.SYNC_METADATA;
          metadata: EmbeddedObjectMetadata;
      }
    | {
          action: EmbeddedUiAction.SET_BLOCK_STYLE;
          style: DocBlockStyle;
      }
    | {
          action: EmbeddedUiAction.ADD_COLUMN;
          before: boolean;
      }
    | {
          action: EmbeddedUiAction.MOVE_LINE;
          direction: EmbeddedMoveDirection;
      }
    | {
          action: EmbeddedUiAction.SORT_ALPHA;
          alphaOrder: boolean;
      }
    | {
          action: EmbeddedUiAction.TAKE_OVER_EDITING;
          editorSession: string;
      }
    | {
          action: EmbeddedUiAction.UPDATE_INLINE_ENTITY_DISPLAY_INFO;
          updatedInfo: InlineEntityDisplayInfo;
      }
    | {
          action: EmbeddedUiAction.COPY_SECTION_LINK;
          onAnchorLinkGenerationSuccessCallback?: (anchorLink: string) => void;
      }
    | {
          action: EmbeddedUiAction.ON_LINK_INSPECTOR_INPUT_CHANGE;
          url: string;
      }
    | {
          action: EmbeddedUiAction.MODIFY_TABLE;
          modification: TableModification;
      };

export declare type AnnotationMessageInfo = {
    count: number;
    seen: boolean;
};

export declare type EmbeddedUiAsyncActionArgs = {
    action: EmbeddedUiAction.UNFURL_URL;
    url: string;
};

export declare type EmbeddedUiData = {
    embeddedObject: () => EmbeddedObject;
    objectUrl: () => string;
    isFocused: () => boolean;
    isLocked: () => boolean;
    isReadOnly: () => boolean;
    isFocusedOrReadOnly: () => boolean;
    isRemovable: () => boolean;
    isDuplicated: () => boolean;
    isCollapsed: () => boolean;
    canToggleCollapse: () => boolean;
    canMoveLine: (direction: EmbeddedMoveDirection) => boolean;
    canAddColumn: () => boolean;
    canAddComment: () => boolean;
    isBookmarked: () => boolean;
    annotationMessageCount: () => number;
    annotationMessageInfo: () => AnnotationMessageInfo;
    hasAnnotationMessages: () => boolean;
    contextType: () => EmbeddedUiContext;
    performAction: (args: EmbeddedUiActionArgs) => void;
    getAssociatedEntityId(): string | null;
    blockStyle: () => DocBlockStyle | undefined;
    lastEdited: () =>
        | {
              lastSlackEditorId?: string;
              lastEditTs: number;
          }
        | undefined;
    hasCheckedItems: () => boolean;
    hasUncheckedItems: () => boolean;
    editingLockSession: () =>
        | {
              lockingAuthorId?: string;
              sessionId: string;
          }
        | undefined;
    associatedInlineEntityDisplayInfo: () => InlineEntityDisplayInfo | null;
    tableSelectionOptions: () => TableSelectionOptions | null;
    tableSectionAttributes: () => TableSectionAttributes | null;
    showAddSectionHighlight: () => boolean;
    preventCardUnfurlConversion: () => boolean;
    getCardRenderType: () => CardRenderType;
    setCardRenderType: (type: CardRenderType) => void;
    performAsyncAction: (
        args: EmbeddedUiAsyncActionArgs
    ) => Promise<string | undefined>;
};

export interface EmbeddedUiProvider {
    embed(
        object: EmbeddedObject,
        container: Element,
        embeddedUiData: EmbeddedUiData
    ): void;
    unmount(container: Element): void;
    focusContent(container: Element, focusEnd?: boolean): void;
}

/** Quip -> host API for opening permalinks */
export interface PermalinkUiProvider {
    /**
     * The user indicated that they want to open an external link (e.g.
     * one embedded in a doc), possibly in a new window. If the host cannot
     * perform this operation, false should be returned.
     */
    openLink?(url: string, options?: OpenLinkOptions): boolean;
    /**
     * The user indicated that they wish to navigate to a document
     * (e.g. because of clicking on a mention). If the host cannot perform
     * this navigation, false is returned.
     */
    openDoc?(id: string, options?: OpenLinkOptions): boolean;
    /**
     * Similar open hooks for Slack entities
     */
    openSlackChannel?(channelId: string, options?: OpenLinkOptions): boolean;
    openSlackUser?(userId: string, options?: OpenLinkOptions): boolean;
}

export interface DocEmbed extends ComponentRef {
    readonly commentsUiProvider?: CommentsUiProvider;
    setCommentsUiProvider(provider?: CommentsUiProvider): void;
    readonly autocompleteUiProvider?: AutocompleteUiProvider;
    setAutocompleteUiProvider(provider?: AutocompleteUiProvider): void;
    readonly permalinkUiProvider?: PermalinkUiProvider;
    setPermalinkUiProvider(provider?: PermalinkUiProvider): void;
    readonly embeddedUiProvider?: EmbeddedUiProvider;
    setEmbeddedUiProvider(provider?: EmbeddedUiProvider): void;
    readonly errorUiProvider?: ErrorUiProvider;
    setErrorUiProvider(errorUiProvider?: ErrorUiProvider): void;
    readonly editorUiConnector?: EditorUiConnector;
    setEditorUiConnector(editorUiConnector?: EditorUiConnector): void;
    readonly scrollPosition: Value<DocScrollPosition>;
    readonly focusState: Value<DocFocusState | null>;
    readonly selectionState: Value<DocSelectionState | null>;
    readonly a11yReadOnlyMode: Value<boolean>;
    readonly docEditor: DocEditor;
    readonly textSectionOverlayShowing: Value<boolean>;
    readonly templateControls: Value<TemplateControlsInfo>;
}

export interface ControllerProviders {
    registerEmbeddedUiProvider(config: EmbeddedUiProviderConfig): void;
    registerAutocompleteTypes(types: SlackObjectType[]): void;
    embeddedUiDidBecomeActive(frameName: string): void;
    exitEmbeddedUi(frameName: string): void;
    getAtomClient(atomStorageProvider?: AtomStorageProvider): AtomClient;
    getCommentsDataProviderDelegate(): CommentsDataProviderDelegate;
    getSlackObjectProviderDelegate(): SlackObjectProviderDelegate;
    getDocumentEventProviderDelegate(): DocumentEventProviderDelegate;
    getBookmarksDataProviderDelegate(): BookmarksDataProviderDelegate;
}

export declare type Appearance =
    | "light"
    | "dark"
    | "light_high_contrast"
    | "dark_high_contrast";

export declare type Language =
    | "en"
    | "es"
    | "de"
    | "fr"
    | "it"
    | "ja"
    | "ko"
    | "nl"
    | "pt"
    | "ru"
    | "tr"
    | "up"
    | "zh_CN";

export interface ControllerSettings {
    setAppearance(value: Appearance): void;
    setLanguage(value: Language): void;
    setUnderlineLinks(value: boolean): void;
    setEnableAnimations(value: boolean): void;
    setDisplayEmojiAsPlainText(value: boolean): void;
    setUseSmartQuotesAndDashes(value: boolean): void;
    setAppKeyboardFocusModeEnabled(value: boolean, activeWindow: Window): void;
    setDefaultTeamId(teamId: string): void;
}

export declare type ControllerObjects = {
    docs: ControllerDocs;
    settings: ControllerSettings;
    providers: ControllerProviders;
};

export declare type JSONValue =
    | string
    | number
    | boolean
    | JSONObject
    | JSONArray;

export interface JSONObject {
    [x: string]: JSONValue;
}

export declare type JSONArray = Array<JSONValue>;

declare type ControllerAccessRun = () => void;

export declare class ControllerAccess {
    get docs(): ControllerDocs;
    get settings(): ControllerSettings;
    get providers(): ControllerProviders;
    /**
     * Run a method accessing controller functionality when it becomes available.
     */
    run(fn: ControllerAccessRun): void;
    setObjects({docs, settings, providers}: ControllerObjects): void;
}

export declare class DocsApi {
    /**
     * Creates a new document. The returned document is optimistically created,
     * and will be saved to the server in the background. The ID will begin with
     * a `temp:` prefix while it's in that state (and `isCreated()` will return
     * false).
     */
    create(options?: DocCreateOptions): Promise<Doc>;
    /** Allows loading of an existing document */
    get(docId: string): Doc;
    get controllerAccess(): ControllerAccess;
    setEditsDisabled(docId: string, disableEdits: boolean): void;
    getBlockKitThumbnail(
        docId: string,
        hideTitle?: boolean,
        sectionId?: string
    ): Promise<BlockKitThumbnail>;
    getCannedDocName(docId: string): Promise<string | null>;
    getSectionSnippet(sectionId: string): Promise<string>;
    /**
     * Allows internal debugging flags to be toggled. Supported keys include:
     * - disable-background-save: Disable automatic saving of the document
     * - enabled: Enable generic debugging logging (required for below keys)
     * - clipboard-events: Log details of cut/copy/paste events
     * - document-updates: Log details of document updates (from saves, other
     *   users, etc.)
     *
     * Warranty may be voided, use at your own risk.
     */
    setDebugOption(key: string, value: boolean): void;
    /**
     * Searches over document sections for a slack object matching type + ID. Returns
     * section ID if any.
     */
    findSlackObject(
        docId: string,
        type: SlackObjectType,
        id?: string
    ): Promise<string | null | undefined>;
    insertSlackObject(
        docId: string,
        object: SlackObject,
        focusedSectionId?: string,
        url?: string,
        isBookmarked?: boolean
    ): Promise<string | null | undefined>;
    editLink(docId: string, controlId: string, link: Link): void;
    /**
     * Load the document contents into the cache
     */
    loadData(docId: string, reason: string): Promise<null | undefined>;
}

export declare type DocCreateOptions = {
    /**
     * Initial title to give to the document (otherwise "Untitled" will be
     * used).
     */
    title?: string;
    cannedDoc?: CreateDocumentOptions["cannedDoc"];
    /**
     * String context to pass through to the backend when creating the document.
     */
    context?: string;
    /**
     * Whether the document should only use inline comments on text
     */
    usesOnlyInlineTextComments?: boolean;
    /**
     * Default content that should be imported into doc when creating.
     * Currently we only support html
     */
    content?: string;
};

export declare type BlockKitThumbnail = {
    blocks?: any[];
    error?: boolean;
};

export declare class Doc {
    /** Document thread ID */
    readonly id: Value<string>;
    readonly title: Value<string>;
    readonly lastUpdated: Value<Date>;
    readonly lastUpdatedUsec: Value<number>;
    readonly lastUpdatedSlackUserId: Value<string>;
    readonly deleted: Value<boolean>;
    readonly isEmpty: Value<boolean | null>;
    readonly hasFileUnfurls: Value<boolean>;
    readonly templateControlsInfo: Value<TemplateControlsInfo>;
    readonly lastNewSectionUpdated: Value<Date>;
    readonly hasHeaderImage: Value<boolean>;
    getBlockKitThumbnailValue(): Value<BlockKitThumbnail>;
    /** Renders the document as a UI component. */
    embed(container: Element, options?: DocEmbedOptions): DocEmbed;
    /**
     * Returns true if the document is fully created (has been persisted to the
     * server).
     */
    isCreated(): boolean;
}

/**
 * Duplicate core/io's PathRewriteHook to avoid importing in core/io
 * directly into embedded code and breaking TypeScript
 */
export declare type PathRewriteHook = (path: string) => string;

export interface EmbeddedConfigProvider {
    config(): EmbeddedUiProviderConfig;
}

export interface UrlParsersProvider {
    /**
     * A list of url parsers(type -> reg) provided by Host
     */
    supportedUrlParsers: UrlTypeToRegExp[];
}

/** Quip -> host API for editing links. */
export interface LinkUiProvider {
    editLink?(docId: string, controlId: string, link: Link): void;
}

export declare type InitConfig = {
    /**
     * Light/dark/high-contrast appearance to use. Can be specified at
     * initialization time and later updated with `setAppearance()`
     */
    appearance?: Appearance;
    /**
     * The language of the user. Can be updated with `setLanguage()`
     */
    language?: Language;
    /**
     * OAuth token for the current user. Meant for development/testing use only,
     * tokens can be obtained via https://quip.com/dev/token.
     * If missing, ambient auth inferred from cookies will be used.
     */
    authToken?: string;
    /**
     * JWT making claims for the Slack user. Used to generate a Quip limited
     * user session token.
     */
    userJwt?: string;
    /**
     * Callback to get token for API request
     */
    getSlackToken?: () => string;
    /**
     * Hook to rewrite request paths
     */
    pathRewriteHook?: PathRewriteHook;
    /**
     * Whether to set withCredential flag for XHR.
     */
    xhrWithCredentials?: boolean;
    /** If true, defaults that requests are CORS requests */
    requestsAreCors?: boolean;
    /** If true quip will enable web caching */
    enableWebCaching?: boolean;
    /** Quip server API origin (defaults to `https://prod.slack.quip.systems`) */
    origin?: string;
    /**
     * Host-provided delegate that allows hooking into SDK lifecycle
     * events and customizing behavior. All methods on the delegate are
     * optional.
     */
    delegate?: PlatformDelegate;
    /**
     * Optional features that the host chooses to enable.
     */
    features?: Feature[];
    /**
     * Provider for persisting values of atoms
     */
    atomStorageProvider?: AtomStorageProvider;
    /**
     * Quip -> host API, to allow the host to create comments thread and other
     * data-related operations.
     */
    commentsDataProvider?: CommentsDataProvider;
    /**
     * Quip -> host API to allow us to react to sections being bookmarked/unbookmarked
     */
    bookmarksDataProvider?: BookmarksDataProvider;
    /**
     * @deprecated - Use docEmbed.setEmbeddedUiProvider
     * Allows the host to render the UI for embedded objects within the SDK
     * UI.
     */
    embeddedUiProvider?: EmbeddedUiProvider & EmbeddedConfigProvider;
    /**
     * Global config for embedded UI
     */
    embeddedConfigProvider?: EmbeddedConfigProvider;
    /** Quip -> host API, to allow file and image upload. */
    fileDataProvider?: FileDataProvider;
    /**
     * Quip -> Slack, to allow the host to provide Slack-related data to power
     * features such as mentions and autocomplete.
     */
    slackObjectProvider?: SlackObjectProvider;
    /** Quip -> host API, to send document data event. */
    documentEventProvider?: DocumentEventProvider;
    /**
     * Flags to Collab SDK that this Collab instance was instantiated from a
     * context that has a Collab instance that can be used for seeding state.
     */
    isChildContext?: boolean;
    /** Slack user accessibility settings preferences. */
    underlineAllLinks?: boolean;
    enableAnimations?: boolean;
    displayEmojiAsPlainText?: boolean;
    /** Quip -> host API, to allow the host provide url parsers */
    urlParsersProvider?: UrlParsersProvider;
    /**
     * Slack-provided team id.  Where this collab was created for use by a
     * particular workspace within an org, this will be that workspace's id.
     * If the collab was created for use by the whole org, it will be the org's
     * id.
     */
    teamId?: string;
    /** Slack-provided top level teamId. */
    topLevelTeamId?: string;
    /** Quip -> host API, to send anchor link data. */
    anchorLinkProvider?: AnchorLinkProvider;
    /**
     * @deprecated No longer used
     * Quip -> host API, to edit links.
     * */
    linkUiProvider?: LinkUiProvider;
    /**
     * Quip -> Slack, to get Slack to parse their own Quill data and give it
     * back in Quip legible HTML.
     */
    slackTextyHtmlConversionProvider?: SlackTextyHtmlConversionProvider;
    /**
     * Quip -> Slack, to share/save slack message objects.
     */
    slackObjectUIProvider?: SlackObjectUIProvider;
    /**
     * Mac's system setting for text
     */
    useSmartQuotesAndDashes?: boolean;
    /**
     * An index, starting at zero, which increments upon each failed attempt
     * to construct a collab SDK.
     */
    attemptNumber?: number;
    /**
     * Whether this client is to be loaded using govslack resources.  Does not
     * necessarily indicate whether the org the client is associated with is a
     * govslack org.  This will be triggered if any of the orgs in the current
     * slack client are govslack.
     */
    isGov?: boolean;
    /**
     * Whether to graft JS/CSS resources for Quip.
     */
    useGraftedMode?: boolean;
    /**
     * Gates whether to use the shared fork feature of SystemJS for ancillary
     * packages.
     */
    useSystemJsSharedFork?: boolean;
};

declare class CollabInstance {
    readonly docs: DocsApi;
    readonly failedToInitialize: Value<boolean>;
    readonly initializationError: Value<string | undefined>;
    constructor(config: InitConfig);
    controllerModuleUrls(): string[];
    preload(): boolean;
    /**
     * Removes controller frame and cleans up event listenrs. Note: View frames
     * must be unmounted before calling tearDown.
     */
    tearDown(): void;
    preloadThreads(threadIds: string[], reason: string): void;
    setAppearance(appearance: Appearance): void;
    setLanguage(language: Language): void;
    setAppKeyboardFocusModeEnabled(
        enabled: boolean,
        activeWindow: Window
    ): void;
    setUnderlineAllLinks(underlineAllLinks: boolean): void;
    setEnableAnimations(enableAnimations: boolean): void;
    setDisplayEmojiAsPlainText(displayEmojiAsPlainText: boolean): void;
    setUseSmartQuotesAndDashes(enable: boolean): void;
    /**
     * @param maxLength maximum length in chars.
     */
    getInitLog(maxLength?: number | undefined): Promise<JSONObject>;
    getInitLogStr(maxLength: number): Promise<string>;
    /** @deprecated */
    getDebugReport(redacted: boolean): JSONObject;
    getBridgeDebugReport(redacted?: boolean): JSONObject;
}

/**
 * Local URLs used for grafting Quip local resources.
 * Should be kept in sync with urls in /static/tools/grafter-v2/*
 */
export declare enum GrafterUrls {
    CONTROLLER_CSS = "https://quip.codes/-/static/less/stylesheet/-?retina=True&mode=normal&less=collab%2Fcontroller.less",
    CONTROLLER_JS = "https://slackdocs.codes:3000/ts/src/collab/controller/bundle.ts",
    EMBEDDED_JS = "https://slackdocs.codes/-/static-script-files?grafted=True&resource_bundle_name=collab_embedded",
}

export {CollabInstance as CollabApi};
